@@ -1,5 +1,5 @@
 /*******************************************************************************#
-#           guvcview              http://guvcview.berlios.de                    #
+#           guvcview              http://guvcview.sourceforge.net               #
 #                                                                               #
 #           Paulo Assis <pj.assis@gmail.com>                                    #
 #                                                                               #
@@ -31,15 +31,13 @@
 /*  Paulo Assis (6-4-2008): removed reading functions, cleaned build wranings  */
 
 #include <stdio.h>
+#include <glib/gstdio.h>
 #include <fcntl.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
 #include <time.h>
-#include <glib.h>
-#include <glib/gstdio.h>
-#include <glib/gprintf.h>
 #include "config.h"
 #include "avilib.h"
 #include "defs.h"
@@ -72,6 +70,8 @@ static char id_str[MAX_INFO_STRLEN];
 #define VERSION "1.0"
 #endif
 
+#define __MUTEX &AVI->mutex
+
 /* AVI_MAX_LEN: The maximum length of an AVI file, we stay a bit below
     the 2GB limit (Remember: 2*10^9 is smaller than 2 GB - using 1900*1024*1024) */
 
@@ -262,17 +262,19 @@ static int avi_add_index_entry(struct avi_t *AVI, BYTE *tag, long flags, long po
 //ThOe write preliminary AVI file header: 0 frames, max vid/aud size
 static int avi_update_header(struct avi_t *AVI)
 {
-	int njunk, sampsize, hasIndex, ms_per_frame, frate, flag;
+	int njunk, sampsize, ms_per_frame, frate, flag;
 	int movi_len, hdrl_start, strl_start, j;
 	BYTE AVI_header[HEADERBYTES];
 	long nhb;
 	ULONG xd_size, xd_size_align2;
 
+	//assume index will be written
+	//int hasIndex=1;
+	
 	//assume max size
 	movi_len = AVI_MAX_LEN - HEADERBYTES + 4;
 
-	//assume index will be written
-	hasIndex=1;
+	
 
 	if(AVI->fps < 0.001) 
 	{
@@ -539,8 +541,9 @@ int AVI_open_output_file(struct avi_t *AVI, const char * filename)
    
 	/*resets AVI struct*/ 
 	memset((void *)AVI,0,sizeof(struct avi_t));
-	/*create a new mutex*/
-	AVI->mutex = g_mutex_new();
+	
+	__INIT_MUTEX(__MUTEX);
+
 	AVI->closed = 0; /*opened - recordind*/
 	
 	/* Since Linux needs a long time when deleting big files,
@@ -553,8 +556,9 @@ int AVI_open_output_file(struct avi_t *AVI, const char * filename)
 	if (AVI->fdes < 0)
 	{
 		AVI_errno = AVI_ERR_OPEN;
-		g_mutex_free(AVI->mutex);
-		//free(AVI);
+		
+		__CLOSE_MUTEX(__MUTEX);
+		
 		return -1;
 	}
 
@@ -569,8 +573,9 @@ int AVI_open_output_file(struct avi_t *AVI, const char * filename)
 		if(fsync(AVI->fdes) || close(AVI->fdes))
 			perror("AVI ERROR: couldn't write to avi file\n");
 		AVI_errno = AVI_ERR_WRITE;
-		g_mutex_free(AVI->mutex);
-		//free(AVI);
+		
+		__CLOSE_MUTEX(__MUTEX);
+		
 		return -2;
 	}
 
@@ -590,7 +595,7 @@ void AVI_set_video(struct avi_t *AVI, int width, int height, double fps, const c
 
 	if(AVI->mode==AVI_MODE_READ) return;
 
-	g_mutex_lock(AVI->mutex);
+	__LOCK_MUTEX(__MUTEX);
 		AVI->width  = width;
 		AVI->height = height;
 		AVI->fps    = fps; /* just in case avi doesn't close properly */
@@ -606,7 +611,7 @@ void AVI_set_video(struct avi_t *AVI, int width, int height, double fps, const c
    
 		AVI->compressor[4] = 0;
 		avi_update_header(AVI);
-	g_mutex_unlock(AVI->mutex);
+	__UNLOCK_MUTEX(__MUTEX);
 
 }
 
@@ -616,7 +621,7 @@ void AVI_set_audio(struct avi_t *AVI, int channels, long rate, int mpgrate, int
 
 	if(AVI->mode==AVI_MODE_READ) return;
 
-	g_mutex_lock(AVI->mutex);
+	__LOCK_MUTEX(__MUTEX);
 		//inc audio tracks
 		AVI->aptr=AVI->anum;
 		++AVI->anum;
@@ -635,7 +640,7 @@ void AVI_set_audio(struct avi_t *AVI, int channels, long rate, int mpgrate, int
 		AVI->track[AVI->aptr].mpgrate = mpgrate; /* bit rate in b/s */
 
 		avi_update_header(AVI);
-	g_mutex_unlock(AVI->mutex);
+	__UNLOCK_MUTEX(__MUTEX);
 }
 
 /*
@@ -1089,7 +1094,7 @@ int AVI_write_frame(struct avi_t *AVI, BYTE *data, long bytes, int keyframe)
 
 	if(AVI->mode==AVI_MODE_READ) { AVI_errno = AVI_ERR_NOT_PERM; return -1; }
 
-	g_mutex_lock( AVI->mutex);
+	__LOCK_MUTEX(__MUTEX);
 		pos = AVI->pos;
 		ret = avi_write_data(AVI,data,bytes,0, keyframe);
    
@@ -1099,7 +1104,7 @@ int AVI_write_frame(struct avi_t *AVI, BYTE *data, long bytes, int keyframe)
 			AVI->last_len = bytes;
 			AVI->video_frames++;
 		}
-	g_mutex_unlock( AVI->mutex);
+	__UNLOCK_MUTEX(__MUTEX);
 	return ret;
 }
 
@@ -1109,11 +1114,11 @@ int AVI_dup_frame(struct avi_t *AVI)
 
 	if(AVI->last_pos==0) return 0; /* No previous real frame */
 
-	g_mutex_lock( AVI->mutex);
+	__LOCK_MUTEX(__MUTEX);
 		if(avi_add_index_entry(AVI,(BYTE *) "00dc",0x10,AVI->last_pos,AVI->last_len)) return -1;
 		AVI->video_frames++;
 		AVI->must_use_index = 1;
-	g_mutex_unlock( AVI->mutex);
+	__UNLOCK_MUTEX(__MUTEX);
 	return 0;
 }
 
@@ -1122,7 +1127,7 @@ int AVI_write_audio(struct avi_t *AVI, BYTE *data, long bytes)
 	int ret=0;
 	if(AVI->mode==AVI_MODE_READ) { AVI_errno = AVI_ERR_NOT_PERM; return -1; }
 
-	g_mutex_lock( AVI->mutex);
+	__LOCK_MUTEX(__MUTEX);
 		ret = avi_write_data(AVI,data,bytes,1,0);
    
 		if(!(ret<0)) 
@@ -1130,7 +1135,7 @@ int AVI_write_audio(struct avi_t *AVI, BYTE *data, long bytes)
 			AVI->track[AVI->aptr].audio_bytes += bytes;
 			AVI->track[AVI->aptr].audio_chunks++;
 		}
-	g_mutex_unlock( AVI->mutex);
+	__UNLOCK_MUTEX(__MUTEX);
 	return ret;
 }
 
@@ -1145,7 +1150,7 @@ int AVI_append_audio(struct avi_t *AVI, BYTE *data, long bytes)
 	if(AVI->mode==AVI_MODE_READ) { AVI_errno = AVI_ERR_NOT_PERM; return -1; }
   
 	// update last index entry:
-	g_mutex_lock( AVI->mutex);
+	__LOCK_MUTEX(__MUTEX);
 		--AVI->n_idx;
 		length = str2ulong(AVI->idx[AVI->n_idx]+12);
 		pos    = str2ulong(AVI->idx[AVI->n_idx]+8);
@@ -1170,7 +1175,7 @@ int AVI_append_audio(struct avi_t *AVI, BYTE *data, long bytes)
 		bytes = i - length;
 		avi_write(AVI->fdes, data, bytes);
 		AVI->pos = pos + 8 + i;
-	g_mutex_unlock( AVI->mutex);
+	__UNLOCK_MUTEX(__MUTEX);
 	
 	return 0;
 }
@@ -1218,7 +1223,7 @@ int AVI_close(struct avi_t *AVI)
 	/* If the file was open for writing, the header and index still have
 	to be written */
 	
-	g_mutex_lock( AVI->mutex);
+	__LOCK_MUTEX(__MUTEX);
 		AVI->closed = 1; /*closed - not recording*/
 		if(AVI->mode == AVI_MODE_WRITE)
 			ret = avi_close_output_file(AVI);
@@ -1235,9 +1240,10 @@ int AVI_close(struct avi_t *AVI)
 			AVI->track[j].audio_bytes=0;  
 			if(AVI->track[j].audio_index) free(AVI->track[j].audio_index);
 		}
-	g_mutex_unlock( AVI->mutex);
+	__UNLOCK_MUTEX(__MUTEX);
 	/*free the mutex*/
-	g_mutex_free(AVI->mutex);
+	__CLOSE_MUTEX(__MUTEX);
+	
 	return ret;
 }
 
