@@ -30,11 +30,13 @@
 #define __AMUTEX &pdata->mutex
 #define __VMUTEX &videoIn->mutex
 #define __GMUTEX &global->mutex
+#define __FMUTEX &global->file_mutex
 #define __GCOND  &global->IO_cond
 
 int initGlobals (struct GLOBAL *global) 
 {
 	__INIT_MUTEX( __GMUTEX );
+	__INIT_MUTEX( __FMUTEX );
 	__INIT_COND( __GCOND );   /* Initialized video buffer semaphore */
 
 	global->debug = DEBUG;
@@ -115,7 +117,8 @@ int initGlobals (struct GLOBAL *global)
 	global->timer_id=0;
 	global->image_timer_id=0;
 	global->image_timer=0;
-	global->image_npics=999;/*default max number of captures*/
+	global->image_npics=9999;/*default max number of captures*/
+	global->image_picn =0;
 	global->frmCount=0;
 	global->PanStep=2;/*2 degree step for Pan*/
 	global->TiltStep=2;/*2 degree step for Tilt*/
@@ -177,6 +180,7 @@ int closeGlobals(struct GLOBAL *global)
 	g_free(global->mode);
 	g_free(global->Sound_IndexDev);
 	__CLOSE_MUTEX( __GMUTEX );
+	__CLOSE_MUTEX( __FMUTEX );
 	__CLOSE_COND( __GCOND );
 	
 	global->videodevice=NULL;
