@@ -32,6 +32,8 @@ typedef struct _sndDev
 	int id;
 	int chan;
 	int samprate;
+	char name[512];
+	char description[256];
 	//PaTime Hlatency;
 	//PaTime Llatency;
 } sndDev;
@@ -48,7 +50,8 @@ typedef struct _VidBuff
 struct GLOBAL 
 {
 	  	
-	__MUTEX_TYPE mutex;            
+	__MUTEX_TYPE mutex;    //global struct mutex
+	__MUTEX_TYPE file_mutex; //video file mutex
 	__COND_TYPE  IO_cond;      //IO thread semaphore
 
 	VidBuff *videoBuff;    //video Buffer
@@ -108,6 +111,7 @@ struct GLOBAL
 	int disk_timer_id;     //timer id for disk check (free space)
 	int image_timer;       //auto image capture time
 	int image_npics;       //number of captures
+	int image_picn;        //capture number
 	int fps;               //fps denominator
 	int fps_num;           //fps numerator (usually 1)
 	int bpp;               //current bytes per pixel
