@@ -604,6 +604,7 @@ static int videoIn_frame_alloca(struct vdIn *vd, int format, int width, int heig
 			vd->framebuffer = g_new0(unsigned char, framebuf_size);
 			break;
 			
+	    case V4L2_PIX_FMT_Y10BPACK:
 	    case V4L2_PIX_FMT_Y16:
 			// alloc a temp buffer for converting to YUYV
 			tmpbuf_size= width * height * 2; // 2 byte per pixel
@@ -952,6 +953,11 @@ static int frame_decode(struct vdIn *vd, int format, int width, int height)
 			grey_to_yuyv(vd->framebuffer, vd->tmpbuffer, width, height);
 			break;
 			
+		case V4L2_PIX_FMT_Y10BPACK:
+			memcpy(vd->tmpbuffer, vd->mem[vd->buf.index],vd->buf.bytesused);
+			y10b_to_yuyv(vd->framebuffer, vd->tmpbuffer, width, height);
+			break;
+		
 	    case V4L2_PIX_FMT_Y16:
 			memcpy(vd->tmpbuffer, vd->mem[vd->buf.index],vd->buf.bytesused);
 			y16_to_yuyv(vd->framebuffer, vd->tmpbuffer, width, height);
@@ -1080,7 +1086,7 @@ int uvcGrab(struct vdIn *vd, int format, int width, int height, int *fps, int *f
 					video_disable(vd);
 					input_set_framerate (vd, fps, fps_num);
 					video_enable(vd);
-					vd->setFPS = 0; /*no need to query and queue buufers*/
+					vd->setFPS = 0; /*no need to query and queue buffers*/
 				}
 				vd->buf.bytesused = v4l2_read (vd->fd, vd->mem[vd->buf.index], vd->buf.length);
 				vd->timestamp = ns_time_monotonic();
@@ -1122,10 +1128,10 @@ int uvcGrab(struct vdIn *vd, int format, int width, int height, int *fps, int *f
 						video_disable(vd);
 						unmap_buff(vd);
 						input_set_framerate (vd, fps, fps_num);
+						vd->setFPS = 0;
 						query_buff(vd);
 						queue_buff(vd);
 						video_enable(vd);
-						vd->setFPS = 0;
 					}
 					else if(vd->setJPEGCOMP) //change jpeg quality/compression in video frame
 					{
@@ -1298,6 +1304,13 @@ input_set_framerate (struct vdIn * device, int *fps, int *fps_num)
 	fd = device->fd;
 
 	device->streamparm.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
+	ret = xioctl(fd, VIDIOC_G_PARM, &device->streamparm);
+	if (ret < 0)
+		return ret;
+
+	if (!(device->streamparm.parm.capture.capability & V4L2_CAP_TIMEPERFRAME))
+		return -ENOTSUP;
+
 	device->streamparm.parm.capture.timeperframe.numerator = *fps_num;
 	device->streamparm.parm.capture.timeperframe.denominator = *fps;
 	 
@@ -1328,6 +1341,7 @@ input_get_framerate (struct vdIn * device, int *fps, int *fps_num)
 
 	fd = device->fd;
 
+	device->streamparm.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
 	ret = xioctl(fd,VIDIOC_G_PARM,&device->streamparm);
 	if (ret < 0) 
 	{
@@ -1335,9 +1349,17 @@ input_get_framerate (struct vdIn * device, int *fps, int *fps_num)
 	} 
 	else 
 	{
-		// it seems numerator is allways 1 but we don't do assumptions here :-)
-		*fps = device->streamparm.parm.capture.timeperframe.denominator;
-		*fps_num = device->streamparm.parm.capture.timeperframe.numerator;
+		if (device->streamparm.parm.capture.capability & V4L2_CAP_TIMEPERFRAME) {
+			// it seems numerator is allways 1 but we don't do assumptions here :-)
+			*fps = device->streamparm.parm.capture.timeperframe.denominator;
+			*fps_num = device->streamparm.parm.capture.timeperframe.numerator;
+		}
 	}
+	
+	if(*fps == 0 )
+		*fps = 1;
+	if(*fps_num == 0)
+		*fps_num = 1;
+
 	return ret;
 }
