@@ -35,6 +35,8 @@
 #define CODEC_FLAG2_INTRA_REFRESH 0
 #endif
 
+#define __FMUTEX &global->file_mutex
+
 static BITMAPINFOHEADER mkv_codecPriv =
 {
 	.biSize = 0x00000028, //40 bytes 
@@ -378,7 +380,7 @@ static vcodecs_data listSupVCodecs[] = //list of software supported formats
 		.fps          = 0,
 		.monotonic_pts= 1,
 		.bit_rate     = 1500000,
-		.qmax         = 21,
+		.qmax         = 51,
 		.qmin         = 10,
 		.max_qdiff    = 4,
 		.dia          = 2,
@@ -389,10 +391,10 @@ static vcodecs_data listSupVCodecs[] = //list of software supported formats
 		.me_sub_cmp   = 3,
 		.last_pred    = 2,
 		.gop_size     = 100,
-		.qcompress    = 0.4,
-		.qblur        = 0.2,
+		.qcompress    = 0.6,
+		.qblur        = 0.5,
 		.subq         = 5,
-		.framerefs    = 2,
+		.framerefs    = 0,
 		.codec_id     = CODEC_ID_H264,
 		.mb_decision  = FF_MB_DECISION_RD,
 		.trellis      = 0,
@@ -509,6 +511,7 @@ static int write_video_data(struct ALL_DATA *all_data, BYTE *buff, int size)
 	
 	int ret =0;
 	
+	__LOCK_MUTEX( __FMUTEX );
 	switch (global->VidFormat)
 	{
 		case AVI_FORMAT:
@@ -527,6 +530,7 @@ static int write_video_data(struct ALL_DATA *all_data, BYTE *buff, int size)
 			
 			break;
 	}
+	__UNLOCK_MUTEX( __FMUTEX );
 	
 	return (ret);
 }
