@@ -1,5 +1,5 @@
 /*******************************************************************************#
-#           guvcview              http://guvcview.berlios.de                    #
+#           guvcview              http://guvcview.sourceforge.net               #
 #                                                                               #
 #           Paulo Assis <pj.assis@gmail.com>                                    #
 #                                                                               #
@@ -67,7 +67,9 @@ typedef struct _vcodecs_data
 	int me_method;            //lavc motion estimation method
 	int mpeg_quant;           //lavc mpeg quantization
 	int max_b_frames;         //lavc max b frames
+	int num_threads;          //lavc num threads
 	int flags;                //lavc flags
+	int monotonic_pts;		  //use monotonic pts instead of timestamp based
 } vcodecs_data;
 
 
