@@ -1,5 +1,5 @@
 /*******************************************************************************#
-#           guvcview              http://guvcview.berlios.de                    #
+#           guvcview              http://guvcview.sourceforge.net               #
 #                                                                               #
 #           Paulo Assis <pj.assis@gmail.com>                                    #
 #                                                                               #
@@ -27,11 +27,11 @@
 
 #include "guvcview.h"
 #include "ms_time.h"
-#include "mp2.h"
 #include "colorspaces.h"
 #include "lavc_common.h"
 #include "video_format.h"
 #include "vcodecs.h"
+#include "acodecs.h"
 #include "defs.h"
 
 
@@ -77,21 +77,14 @@ gboolean isLavfFormat(int codec_ind)
 	return (listSupVFormats[codec_ind].avformat);
 }
 
-char *setVidExt(char *filename, int format_ind)
-{
-	int sname = strlen(filename)+1; /*include '\0' terminator*/
-	char basename[sname];
-	sscanf(filename,"%[^.]",basename);
-	
-	g_snprintf(filename, sname, "%s.%s", basename, get_vformat_extension(format_ind));
-	
-	return (filename);
-}
-
 int write_video_packet (BYTE *picture_buf, int size, int fps, struct VideoFormatData* videoF)
 {
 	int64_t t_stamp = (int64_t) videoF->vpts; 
 	videoF->b_writing_frame = 0;
+	
+	if( mk_addFrameData(videoF->mkv_w, picture_buf , size) < 0 )
+		return -1;
+		
 	mk_setFrameFlags( videoF->mkv_w, t_stamp, videoF->keyframe );
 	
 	if (!videoF->b_writing_frame) //sort of mutex?
@@ -100,16 +93,18 @@ int write_video_packet (BYTE *picture_buf, int size, int fps, struct VideoFormat
 			return -1;
 		videoF->b_writing_frame = 1;
 	}
-	if( mk_addFrameData(videoF->mkv_w, picture_buf , size) < 0 )
-		return -1;
 
 	return (0);
 }
 
-int write_audio_packet (BYTE *audio_buf, int size, int samprate, struct VideoFormatData* videoF)
+int write_audio_packet (BYTE *audio_buf, int size, struct VideoFormatData* videoF)
 {
 	int64_t t_stamp = (int64_t) videoF->apts ;
 	videoF->b_writing_frame = 0;
+	
+	if( mk_addAudioFrameData(videoF->mkv_w, audio_buf , size) < 0 )
+		return -1;
+		
 	mk_setAudioFrameFlags( videoF->mkv_w, t_stamp, videoF->keyframe );
 
 	if (!videoF->b_writing_frame) //sort of mutex?
@@ -118,8 +113,6 @@ int write_audio_packet (BYTE *audio_buf, int size, int samprate, struct VideoFor
 			return -1;
 		videoF->b_writing_frame = 1;
 	}
-	if( mk_addAudioFrameData(videoF->mkv_w, audio_buf , size) < 0 )
-		return -1;
 
 	return (0);
 
@@ -135,51 +128,22 @@ int clean_FormatContext (void* data)
 	/*------------------- close audio stream and clean up -------------------*/
 	if (global->Sound_enable > 0) 
 	{
-		/*wait for audio to finish*/
-		int stall = wait_ms( &pdata->streaming, FALSE, 10, 30 );
-		if(!(stall)) 
-		{
-			g_printerr("video_format.c(144):sound capture stall (still streaming(%d) \n",
-				pdata->streaming);
-			pdata->streaming = 0;
-		}
-		/*write any available audio data*/
-		g_mutex_lock( pdata->mutex);
-			if(pdata->audio_flag)
-			{
-				g_printerr("writing %d bytes of audio data\n",pdata->snd_numBytes);
-				if(global->Sound_Format == PA_FOURCC)
-				{
-					if(pdata->vid_sndBuff) 
-					{
-						Float2Int16(pdata);
-						write_audio_packet ((BYTE *) pdata->vid_sndBuff1, pdata->snd_numSamples*2, pdata->samprate, videoF);
-					}
-				}
-				else if (global->Sound_Format == ISO_FORMAT_MPEG12)
-				{
-					int size_mp2=0;
-					if(pdata->vid_sndBuff && pdata->mp2Buff) 
-					{
-						size_mp2 = MP2_encode(pdata,0);
-						write_audio_packet (pdata->mp2Buff, size_mp2, pdata->samprate, videoF);
-					}
-				}
-			}
-			pdata->audio_flag = 0; /*all audio should have been writen by now*/
-		g_mutex_unlock( pdata->mutex );
-		if(global->debug) g_printf("closing sound...\n");
+		if(global->debug) g_print("closing sound...\n");
 		if (close_sound (pdata)) g_printerr("Sound Close error\n");
-		if(global->Sound_Format == ISO_FORMAT_MPEG12) close_MP2_encoder();
-		if(global->debug) g_printf("sound closed\n");
+		if(global->debug) g_print("sound closed\n");
 	} 
-	
+
+	int64_t def_duration = ((global->Vidstoptime - global->Vidstarttime)/global->framecount);
+	g_print("video default duration:%llu start:%llu stop:%llu\n", 
+		(unsigned long long) def_duration, (unsigned long long) global->Vidstarttime, 
+		(unsigned long long) global->Vidstoptime);
+	mk_setDef_Duration(videoF->mkv_w, def_duration);/* set real fps ( average frame duration)*/
 	
 	ret = mk_close(videoF->mkv_w);
 	videoF->mkv_w = NULL;
 	videoF->apts = 0;
 	videoF->old_apts = 0;
-	if(global->debug) g_printf("finished cleaning up\n");
+	if(global->debug) g_print("finished cleaning up\n");
 	return (ret);
 }
 
@@ -191,60 +155,75 @@ int init_FormatContext(void *data)
 	struct VideoFormatData *videoF = all_data->videoF;
 	struct paRecordData *pdata = all_data->pdata;
 	
-	char *AcodecID = NULL;
 	int bitspersample = 0;
-	float samprate = -1;
+	float samprate = 16000.0; //reference
 	int channels = 1;
-	int64_t duration = 0;
+	UINT64 duration = 0;
 	
 	if(videoF->mkv_w !=NULL )
 	{
-		g_printf("matroska: older mux ref not closed, cleaning now...\n");
+		g_print("matroska: older mux ref not closed, cleaning now...\n");
 		mk_close(videoF->mkv_w);
 		videoF->mkv_w = NULL;
 	}
 	
 	videoF->mkv_w = mk_createWriter( videoIn->VidFName );
-
+	if(videoF->mkv_w == NULL)
+	{
+		g_printerr("matroska: failed to initialize file\n");
+		return (-1);
+	}
+	
 	if(global->Sound_enable > 0)
 	{
-		switch (global->Sound_Format)
-		{
-			case PA_FOURCC: //pcm codec
-				AcodecID = "A_PCM/INT/LIT";
-				bitspersample = 16;
-				break;
-			case ISO_FORMAT_MPEG12:
-				AcodecID = "A_MPEG/L2";
-				break;
-		}
+		bitspersample = get_aud_bits(get_ind_by4cc(global->Sound_Format));
+	
 		if (pdata) 
 		if(pdata->samprate > 0)
 		{
 			samprate = (float) pdata->samprate;
 			channels = pdata->channels;
 			if(pdata->api == PORT)
-				duration = (int64_t) 1000000000/((samprate * pdata->channels)/(pdata->tresh + (pdata->channels * pdata->MPEG_Frame_size)));
+				duration = (UINT64) (1000*pdata->aud_numSamples)/(pdata->samprate * channels);
 			else
-				duration = (int64_t) 1000000000/((samprate * pdata->channels)/pdata->tresh);
+				duration = (UINT64) (1000*pdata->aud_numSamples)/(pdata->samprate * channels);
+
+			duration = duration * 1000000; //from milisec to nanosec
 		}
+	    
+		if(global->debug) g_print("audio frame: %i | %i | %i | %llu\n", 
+			pdata->aud_numSamples, pdata->samprate, channels, (unsigned long long) duration);
 	}
+	else
+		samprate = 0.0;
 	
 	videoF->apts = 0;
 	videoF->old_apts = 0;
     
-	set_mkvCodecPriv(global->VidCodec, videoIn->width, videoIn->height);
-	int size = set_mkvCodecPriv(global->VidCodec, videoIn->width, videoIn->height);
+	//set_mkvCodecPriv(global->VidCodec, global->width, global->height);
+	unsigned size = set_mkvCodecPriv(global->VidCodec, global->width, global->height);
+	
+	unsigned asize = set_mkvACodecPriv(get_ind_by4cc(global->Sound_Format), pdata->samprate, channels);
+	
 	printf("writing header\n");
+	
+	/*gspca doesn't set the fps value so we don't set it in the file header    */
+	/*this is OK acording to the standard (variable fps )but vlc seems to have */
+	/*a problem with this in the case of mpeg codecs (mpg1 and mpg2)            */
+	UINT64 v_def_dur = 0;
+	if(global->fps >= 5) v_def_dur = (UINT64) (global->fps_num * 1e9/global->fps); //nano seconds
+	if (global->debug) g_print("video frame default duration =%llu for %i fps\n", 
+		(unsigned long long) v_def_dur, global->fps);
 	mk_writeHeader( videoF->mkv_w, "Guvcview",
                      get_mkvCodec(global->VidCodec),
-                     AcodecID,
+                     get_mkvACodec(get_ind_by4cc(global->Sound_Format)),
                      get_mkvCodecPriv(global->VidCodec), size,
-                     (int64_t) (global->fps_num * 1000000000/global->fps), //nano seconds
-                     duration, //FIXME: 1152 samples is a MP2 Frame
+                     v_def_dur,
+                     get_mkvACodecPriv(get_ind_by4cc(global->Sound_Format)), asize,
+                     duration,
                      1000000,
-                     videoIn->width, videoIn->height,
-                     videoIn->width, videoIn->height,
+                     global->width, global->height,
+                     global->width, global->height,
                      samprate, channels, bitspersample );
 	videoF->b_header_written = 1;
 	return(0);
